/*
 *
 * ATM Switch Viewer - SWView
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swview_conf.c,v 1.2 1997/12/18 01:02:52 miwa Exp $
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <stdio.h>

#include "X11/forms.h"
#include "swctl.h"
#include "swview_fd.h"
#include "swview.h"

extern swview_conf_table_t *swview_conf_list;

int swview_conf_parse(char *conf_file)
{
    int line, op;
    int sw_flag = 0;

    FILE *conf_FILE;

    char hoge_buf[BUFSIZ];
    char *hoge_argv[16];
    int	buf_p;
    int	argv_p;

    swview_conf_table_t	*swview_conf_tmp;
    swview_conf_port_t 	*swview_port_tmp;
    swview_conf_path_t 	*swview_path_tmp;
    swview_conf_upc_t  	*swview_upc_tmp;

    swview_conf_list = (swview_conf_table_t *)NULL;
    
    if(!(conf_FILE = fopen(conf_file, "r"))) {
	fprintf(stderr, "swview: %s not found.(ignore)\n", conf_file);
	return(0);
    }

    for( ; ; ) {

	buf_p = 0;
	argv_p = 0;

	if( fgets(hoge_buf, sizeof(hoge_buf), conf_FILE) == NULL )
	  break;

	line++;
	
	while( hoge_buf[buf_p] != '\n' ) {

	    for( ; hoge_buf[buf_p] == ' ' || hoge_buf[buf_p] == '\t';
		buf_p++);

	    if( hoge_buf[buf_p] == '\n' )
	      break;

	    if( hoge_buf[buf_p] == '"' ) {
		hoge_argv[argv_p++] = hoge_buf+buf_p+1;
		buf_p++;
		for( ; hoge_buf[buf_p] != '"' && hoge_buf[buf_p] != '\n';
		    buf_p++ );
	    }
	    else {
		hoge_argv[argv_p++] = hoge_buf+buf_p;
		for( ; hoge_buf[buf_p] != ' ' && hoge_buf[buf_p] != '\t'
		    && hoge_buf[buf_p] != '\n'; buf_p++);
	    }

	    if (hoge_buf[buf_p] == '\n') {
		hoge_buf[buf_p] = '\0';
		break;
	    }
	    hoge_buf[buf_p++] = '\0';
	}

        if ( argv_p == 0 )		/* space line */
	  continue;

	if( hoge_argv[0][0] == '#')	/* comment line */
	  continue;

        op = str2op(hoge_argv[0]);	/* operation parsing */

	if ( op < 0 ) {			/* unknown opertion */
	    fprintf(stdout, "line: %d: op: %s is not defined.(ignore)\n",
		    line, hoge_argv[0]);
	    continue;
	}

	if ( op > 0 && sw_flag == 0 ) {	/* switch not specified */
	    fprintf(stdout, "line: %d: op: %s ... target switch not specified.(ignore)\n",
		    line, hoge_argv[0]);
	    continue;
	}

	switch(op) {

	  case SWVIEW_CONF_SWITCH:
	    if( argv_p < 2 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		break;
	    }
	    
	    swview_conf_tmp = (swview_conf_table_t
			       *)malloc(sizeof(swview_conf_table_t)); 
	    memset(swview_conf_tmp, 0, sizeof(swview_conf_table_t));
	    swview_conf_tmp->next_table = swview_conf_list;
	    swview_conf_tmp->sw =
	      (switch_id_t)inet_addr(hoge_argv[1]);
	    swview_conf_tmp->port = (swview_conf_port_t *)NULL;
	    swview_conf_tmp->path = (swview_conf_path_t *)NULL;
	    swview_conf_tmp->upc = (swview_conf_upc_t *)NULL;
	    swview_conf_list = swview_conf_tmp;

	    sw_flag = 1;
	    break;

	  case SWVIEW_CONF_PORT:
	    if( argv_p < 3 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		break;
	    }

	    swview_port_tmp = (swview_conf_port_t
			       *)malloc(sizeof(swview_conf_port_t));
	    swview_port_tmp->next = swview_conf_list->port;
	    swview_port_tmp->vport = search_vport(hoge_argv[1]);
	    memcpy(swview_port_tmp->peer, hoge_argv[2], 12*sizeof(char));
	    swview_port_tmp->peer[12] = '\0';
	    swview_conf_list->port = swview_port_tmp;
	    
	    break;

	  case SWVIEW_CONF_PATH:
	    if( argv_p < 6 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		break;
	    }

	    swview_path_tmp = (swview_conf_path_t
			       *)malloc(sizeof(swview_conf_path_t));
	    swview_path_tmp->next = swview_conf_list->path;
	    swview_path_tmp->in = search_vport(hoge_argv[1]);
	    swview_path_tmp->out = search_vport(hoge_argv[2]);
	    swview_path_tmp->priority = atoi(hoge_argv[3]);
	    memcpy(swview_path_tmp->color, hoge_argv[4], 4*sizeof(char));
	    swview_path_tmp->color[4] = '\0';
	    memcpy(swview_path_tmp->status, hoge_argv[5], 32*sizeof(char));
	    swview_path_tmp->status[32] = '\0';
	    swview_conf_list->path = swview_path_tmp;
	    
	    break;

	  case SWVIEW_CONF_UPC:
	    if( argv_p < 5 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		break;
	    }

	    swview_upc_tmp = (swview_conf_upc_t
			       *)malloc(sizeof(swview_conf_upc_t));
	    swview_upc_tmp->next = swview_conf_list->upc;
	    swview_upc_tmp->type = str2upctype(hoge_argv[1]);
	    swview_upc_tmp->priority = atoi(hoge_argv[2]);
	    memcpy(swview_upc_tmp->color, hoge_argv[3], 4*sizeof(char));
	    swview_upc_tmp->color[4] = '\0';
	    memcpy(swview_upc_tmp->status, hoge_argv[4], 32*sizeof(char));
	    swview_upc_tmp->status[32] = '\0';
	    swview_conf_list->upc = swview_upc_tmp;
	    
	    break;

	  case SWVIEW_CONF_COMMUNITY:
	    if( argv_p < 3 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		break;
	    }
	    strcpy(swview_conf_tmp->set_community, hoge_argv[1]);
	    strcpy(swview_conf_tmp->get_community, hoge_argv[2]);

	    break;

	  default:

	    break;
				
	}
    }

#ifdef	DEBUG
    for(swview_conf_tmp = swview_conf_list; swview_conf_tmp;
	swview_conf_tmp = swview_conf_tmp->next_table) {
	fprintf(stderr, "table: %s\n",
		inet_ntoa(swview_conf_tmp->sw));
	for(swview_port_tmp = swview_conf_tmp->port; swview_port_tmp;
	    swview_port_tmp = swview_port_tmp->next) {
	    fprintf(stderr, "\tport: %d %s\n",
		    swview_port_tmp->vport,
		    swview_port_tmp->peer);
	}

	for(swview_path_tmp = swview_conf_tmp->path; swview_path_tmp;
	    swview_path_tmp = swview_path_tmp->next) {
	    fprintf(stderr, "\tpath: %d %d %d %s %s\n",
		    swview_path_tmp->in,
		    swview_path_tmp->out,
		    swview_path_tmp->priority,	
		    swview_path_tmp->color,	
		    swview_path_tmp->status);
	}

	for(swview_upc_tmp = swview_conf_tmp->upc; swview_upc_tmp;
	    swview_upc_tmp = swview_upc_tmp->next) {
	    fprintf(stderr, "\tupc: %d %d %s %s\n",
		    swview_upc_tmp->type,
		    swview_upc_tmp->priority,	
		    swview_upc_tmp->color,	
		    swview_upc_tmp->status);
	}
	fprintf(stderr, "\n");
    }
#endif	/* DEBUG */
}

int str2op(char *str)
{
    /* match prefix 3 letters --> yes */

    if(!memcmp((void *)str, "switch", 3*sizeof(char)))
      return(SWVIEW_CONF_SWITCH);

    if(!memcmp((void *)str, "port", 3*sizeof(char)))
      return(SWVIEW_CONF_PORT);

    if(!memcmp((void *)str, "path", 3*sizeof(char)))
      return(SWVIEW_CONF_PATH);
    
    if(!memcmp((void *)str, "upc", 3*sizeof(char)))
      return(SWVIEW_CONF_UPC);

    if(!memcmp((void *)str, "community", 3*sizeof(char)))
      return(SWVIEW_CONF_COMMUNITY);

    return(-1);		/* default */
}

int str2upctype(char *str)
{
    /* match prefix 3 letters --> yes */

    if(!memcmp((void *)str, "PCR01", 5*sizeof(char)))
      return(SWVIEW_CONF_UPC_PCR01);

    if(!memcmp((void *)str, "SCR01", 5*sizeof(char)))
      return(SWVIEW_CONF_UPC_SCR01);

    if(!memcmp((void *)str, "MBS01", 5*sizeof(char)))
      return(SWVIEW_CONF_UPC_MBS01);

    if(!memcmp((void *)str, "PCR0", 4*sizeof(char)))
      return(SWVIEW_CONF_UPC_PCR0);

    if(!memcmp((void *)str, "SCR0", 4*sizeof(char)))
      return(SWVIEW_CONF_UPC_SCR0);

    if(!memcmp((void *)str, "MBS0", 4*sizeof(char)))
      return(SWVIEW_CONF_UPC_MBS0);
    
    if(!memcmp((void *)str, "CDVT", 4*sizeof(char)))
      return(SWVIEW_CONF_UPC_CDVT);

    if(!memcmp((void *)str, "ACT", 3*sizeof(char)))
      return(SWVIEW_CONF_UPC_ACT);

    if(!memcmp((void *)str, "AAL5EPD", 7*sizeof(char)))
      return(SWVIEW_CONF_UPC_AAL5EPD);

    return(-1);		/* default */
}

char *search_portname(int vport)
{
    int i;

    for(i = 0; port_table[i].vport != -1; i++)
      if( port_table[i].vport == vport )
	break;

    return(port_table[i].port);
}

int search_vport(char *portname)
{
    int i;

    for(i = 0; port_table[i].vport != -1; i++)
      if(!(memcmp((void*)portname, (void*)(port_table[i].port),
		  sizeof(portname))))
	break;

    return(port_table[i].vport);
}

char *search_peername(switch_id_t sw, int vport)
{
    swview_conf_table_t *conf_table;
    swview_conf_port_t *conf_port;

    for( conf_table = swview_conf_list; conf_table; conf_table =
	conf_table->next_table )
      if( conf_table->sw == sw )
	break;

    if( conf_table )
      for(conf_port = conf_table->port; conf_port; conf_port =
	  conf_port->next)
	if(conf_port->vport == vport)
	  return(conf_port->peer);

    return(search_portname(vport));
}

int search_peername_rev(switch_id_t sw, char *peername)
{
    swview_conf_table_t *conf_table;
    swview_conf_port_t *conf_port;

    for( conf_table = swview_conf_list; conf_table; conf_table =
	conf_table->next_table )
      if( conf_table->sw == sw )
	break;

    if( conf_table )
      for(conf_port = conf_table->port; conf_port; conf_port =
	  conf_port->next)
	if(!memcmp(conf_port->peer, peername, strlen(peername)))
	  return(conf_port->vport);

    return(search_vport(peername));
}

u_char *search_set_community(switch_id_t sw)
{
    swview_conf_table_t *conf_table;

    for( conf_table = swview_conf_list; conf_table; conf_table =
	conf_table->next_table ) {
	if( conf_table->sw == sw ) {
	    return(conf_table->set_community);
	    break;
	}
    }
    
    return((u_char *)NULL);
}

u_char *search_get_community(switch_id_t sw)
{
    swview_conf_table_t *conf_table;

    for( conf_table = swview_conf_list; conf_table; conf_table =
	conf_table->next_table ) {
	if( conf_table->sw == sw ) {
	    return(conf_table->get_community);
	    break;
	}
    }
    
    return((u_char *)NULL);
}


swview_vccline_t *swview_set_vccline_list(switch_id_t sw,
					  FD_swview_switch *fd_swview_switch,
					  swctl_vcc_table_t *vcc_table_list)
{
    char ctrlcode_buf[BUFSIZ];
    char status_buf[BUFSIZ];

    swview_vccline_t *swview_vccline_list = (swview_vccline_t *)NULL;
    swview_vccline_t *swview_vccline;
    swview_conf_table_t *conf_table;
    swview_conf_port_t *conf_port;
    swview_conf_path_t *conf_path;
    swview_conf_upc_t *conf_upc;

    swctl_vcc_table_t *vcc_table;

    for(conf_table = swview_conf_list; conf_table;
	conf_table = conf_table->next_table) {
	if(conf_table->sw == sw) {
	    break;
	}
    }

    for(vcc_table = vcc_table_list; vcc_table;
	vcc_table = vcc_table->next_table) {

	swview_vccline = (swview_vccline_t
			  *)malloc(sizeof(swview_vccline_t));
	swview_vccline->next_line = swview_vccline_list;

	/* copy vcc_table */
	swview_vccline->vcc_table = copy_vcc_table(vcc_table);

	/* vcc classification */

	if(conf_table) {

	    for( conf_path = conf_table->path; conf_path; conf_path =
		conf_path->next ) {

		if((ADDR2VPORT(vcc_table->in) == conf_path->in
		    || -1 == conf_path->in)
		   && (ADDR2VPORT(vcc_table->out) == conf_path->out
		       || -1 == conf_path->out))
		  break;
	    }
	    
	    if( vcc_table->param ) {


		for( conf_upc = conf_table->upc; conf_upc; conf_upc =
		    conf_upc->next ) {

		    if(conf_upc->type == SWVIEW_CONF_UPC_PCR01
		       && swview_vccline->vcc_table->param->pcr01)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_SCR01
			    && swview_vccline->vcc_table->param->scr01)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_MBS01
			    && swview_vccline->vcc_table->param->mbs01)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_PCR0
			    && swview_vccline->vcc_table->param->pcr0)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_SCR0
			    && swview_vccline->vcc_table->param->scr0)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_MBS0
			    && swview_vccline->vcc_table->param->mbs0)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_CDVT
			    && swview_vccline->vcc_table->param->cdvt)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_ACT
			    && swview_vccline->vcc_table->param->act)
		      break;
		    else if(conf_upc->type == SWVIEW_CONF_UPC_AAL5EPD
			    && swview_vccline->vcc_table->param->aal5epd)
		      break;
		}
	    }
	    else {
		conf_upc = (swview_conf_upc_t *)NULL;
	    }
	    
	    if(conf_upc) {

		swview_vccline->priority = conf_upc->priority;

		/* fill strings */
		sprintf(swview_vccline->str, "%s%10s%6u%6u%12s%6u%6u\t%s",
			conf_upc->color,
			search_peername(sw, ADDR2VPORT(vcc_table->in)),
			ADDR2VPI(vcc_table->in),
			ADDR2VCI(vcc_table->in),
			search_peername(sw, ADDR2VPORT(vcc_table->out)),
			ADDR2VPI(vcc_table->out),
			ADDR2VCI(vcc_table->out),
			conf_upc->status);
	    }

	    else if(conf_path) {

		swview_vccline->priority = conf_path->priority;

		/* fill strings */
		sprintf(swview_vccline->str, "%s%10s%6u%6u%12s%6u%6u\t%s",
			conf_path->color,
			search_peername(sw, ADDR2VPORT(vcc_table->in)),
			ADDR2VPI(vcc_table->in),
			ADDR2VCI(vcc_table->in),
			search_peername(sw, ADDR2VPORT(vcc_table->out)),
			ADDR2VPI(vcc_table->out),
			ADDR2VCI(vcc_table->out),
			conf_path->status);
	    }
	    else {
		  
		/* fill strings */
		sprintf(swview_vccline->str, "%10s%6u%6u%12s%6u%6u",
			search_peername(sw, ADDR2VPORT(vcc_table->in)),
			ADDR2VPI(vcc_table->in),
			ADDR2VCI(vcc_table->in),
			search_peername(sw, ADDR2VPORT(vcc_table->out)),
			ADDR2VPI(vcc_table->out),
			ADDR2VCI(vcc_table->out));

		swview_vccline->priority = MAX_PRIORITY;

	    }
	}
	else {
	    /* fill strings */

	    swview_vccline->priority = MAX_PRIORITY;

	    sprintf(swview_vccline->str, "%10s%6u%6u%12s%6u%6u",
		    search_portname(ADDR2VPORT(vcc_table->in)),
		    ADDR2VPI(vcc_table->in),
		    ADDR2VCI(vcc_table->in),
		    search_portname(ADDR2VPORT(vcc_table->out)),
		    ADDR2VPI(vcc_table->out),
		    ADDR2VCI(vcc_table->out));
		
	}
	
	swview_vccline_list = swview_vccline;
    }

    return(swview_vccline_list);
}
