/*
 *
 * ATM Switch Viewer - SWView
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swview_main.c,v 1.4 1998/04/09 09:09:08 miwa Exp $
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>

#include "X11/forms.h"
#include "swctl.h"
#include "swview_fd.h"
#include "swview.h"

static  char SWVIEW_VERSION[] = "swview_version_0.2.4";

swview_window_t	*swview_window_list = (swview_window_t *)NULL;
swview_status_t *swview_status_list = (swview_status_t *)NULL;
swview_conf_table_t *swview_conf_list = (swview_conf_table_t *)NULL;

FD_swview_main *fd_swview_main;

void
swview_open_act(FL_OBJECT *ob, long data)
{
    void *ptr;
    int val;
    char notion_buf[BUFSIZ];

    struct hostent hostentry;

    swview_window_t *swview_window;
    swview_vccline_t *swview_vccline_list;

    switch_id_t	sw;
    swctl_vcc_table_t *vcc_table_list = (swctl_vcc_table_t *)NULL;
    swctl_vcc_table_t *vcc_table;

    fl_deactivate_all_forms();
    
    sprintf(notion_buf, "Open ATM Switch: %s ?", fl_get_input(ob));

    if( swview_yesno_show(notion_buf) ) {

	/* cannot resolv hostname */
	if( (struct hostent *)NULL ==
	   (ptr = (struct hostent *)gethostbyname(fl_get_input(ob)))) {
	    sprintf(notion_buf, "Unknow Switch: %s", fl_get_input(ob));
	    swview_alert_show(notion_buf);
	}

	else {
	    hostentry = *(struct hostent *)ptr;
	    sw = (*((struct in_addr *)(hostentry.h_addr))).s_addr;

	    /* the switch is already open */
	    if( swview_search_window(sw) ) {
		sprintf(notion_buf, "Switch %s is ALREADY open.",
			hostentry.h_name);
		swview_alert_show(notion_buf);
	    }

	    else  {

		/* swctl_open error */
		if( SNMP_SUCCESS !=
		   (val = swctl_open(sw,
				     search_set_community(sw),
				     search_get_community(sw),
				     &vcc_table_list)) ) {
		    swview_swctl_error(val, hostentry.h_name);
		}

		else {
		    /* make swview_window entry */
		    swview_window = (swview_window_t
				     *)malloc(sizeof(swview_window_t));
		    swview_window->next_window = swview_window_list;
		    swview_window->sw = sw;
		    swview_window->name = (char
					   *)malloc((strlen(hostentry.h_name)
						     +1)*sizeof(char));
		    memcpy((void *)swview_window->name,
			   (void *)(hostentry.h_name),
			    (strlen(hostentry.h_name)+1)*sizeof(char)); 
		    swview_window->window = create_form_swview_switch();
		    swview_window_list = swview_window;

		    /* set attribute fd_swview_switch */
		    sprintf(notion_buf, "%s (%s)",
			    hostentry.h_name, inet_ntoa(sw));
		    fl_set_object_label(swview_window->window
					->swview_switchname_text,
					notion_buf);

		    fl_set_browser_fontsize(swview_window->window
					    ->swview_vcc_browser, 12);
		    fl_set_browser_fontstyle(swview_window->window
					    ->swview_vcc_browser, 5);

		    /* show fd_swview_switch */
		    fl_show_form(swview_window->window->swview_switch,
				 FL_PLACE_SIZE, FL_FULLBORDER,
				 hostentry.h_name);

		    /* make vccline_list */
		    swview_window->vccline_list = 
		      swview_set_vccline_list(swview_window->sw,
					      swview_window->window,
					      vcc_table_list);

		    /* fill fd_swview_switch->swview_vcc_browser */
		    fl_freeze_form(swview_window->window->swview_switch);

		    swview_fill_vccline(swview_window->sw,
					swview_window->window,
					swview_window->vccline_list);

		    fl_unfreeze_form(swview_window->window->swview_switch);

		    /* clear fd_swview_switch->swview_open_input */
		    fl_set_input(ob, "");

		    free_vcc_table_list(vcc_table_list);

		}
	    }	
	}
    }

    fl_activate_all_forms();
}

void
swview_exit_act(FL_OBJECT *ob, long data)
{
    exit(0);
}

void
swview_vcc_act(FL_OBJECT *ob, long data)
{

    /* which button is pressed ? */

#ifdef	XFORMS081

    XEvent e;
    memcpy((void *)&e, (void *)fl_last_event(), sizeof(XEvent));
    if(e.type == ButtonRelease) {
	switch (e.xbutton.button) {
	  case 1:
	    swview_mon_act(ob, data);
	    break;
	    
	  case 2:
	    break;

	  case 3:
	    swview_operation_act(ob, data);
	    break;
	    
	  default:
	    break;
	}
    }

#else	/* XFORMS086 ... default */

    switch (fl_mouse_button()) {
      case FL_LEFT_MOUSE:
	swview_mon_act(ob, data);
	break;
	
      case FL_MIDDLE_MOUSE:
	break;
	
      case FL_RIGHT_MOUSE:
	swview_operation_act(ob, data);
	break;
	    
      default:
	break;
    }

#endif	/* XFORMS081 */
}

void
swview_mon_act(FL_OBJECT *ob, long data)
{
    int i, line;
    char notion_buf[BUFSIZ];
    char output_buf[BUFSIZ];

    swview_window_t *swview_window;
    swview_vccline_t *swview_vccline;
    swview_status_t *swview_status;

    swctl_vcc_table_t *vcc_table;
    swctl_param_t *param;

    /* search parent window object */
    swview_window = swview_search_parent_window(ob);

    /* search vccline object */
    line = fl_get_browser(ob);
    for(swview_vccline = swview_window->vccline_list; swview_vccline;
	swview_vccline = swview_vccline->next_line) {
	if(swview_vccline->line == line) {
	    vcc_table = swview_vccline->vcc_table;
	    break;
	}
    }

    /* the vcc is already monitoring */
    if( swview_search_status(swview_window->sw, vcc_table) ) {
	sprintf(notion_buf, "That VCC is ALREADY monitoring.");
	swview_alert_show(notion_buf);
    }

    else {

	fl_deactivate_all_forms();

	/* make swview_status entry */
	swview_status = (swview_status_t *)malloc(sizeof(swview_status_t));
	swview_status->next_status = swview_status_list;
	swview_status->window = swview_window;
	swview_status->vcc_table = copy_vcc_table(vcc_table);
	swview_status->status = create_form_swview_status();

	for(i = 0; i < POLLING_ITEMS; i++) {
	    swview_status->items[i] = 0;
	    swview_status->times[i] = -1;
	    fl_add_chart_value(swview_status->status->swview_traffic_chart,
			       swview_status->items[i], "", 1);
	}
	swctl_mon(swview_window->sw, vcc_table->in, vcc_table->out,
		  &(swview_status->cells));
	swview_status->max_cells = 0;
	swview_status->timer = 0;

	swview_status_list = swview_status;

	/* set attribute fd_swview_status */
	sprintf(output_buf,
		"INPUT:\n\t\t%s\t\t%d\t\t%d\nOUTPUT:\n\t\t%s\t\t%d\t\t%d", 
		search_portname(ADDR2VPORT(vcc_table->in)),
		ADDR2VPI(vcc_table->in), ADDR2VCI(vcc_table->in),
		search_portname(ADDR2VPORT(vcc_table->out)),
		ADDR2VPI(vcc_table->out), ADDR2VCI(vcc_table->out));

	fl_set_object_label(swview_status->status->swview_vcc_text, output_buf);

	if(vcc_table->param) {
	    sprintf(output_buf,
		    "\nPCR01\t%u\nSCR01\t%u\nMBS01\t%u\nPCR0\t%u\nSCR0\t%u\nMBS0\t%u\nCDVT\t%u\nACT\t%s\nAAL5EPD\t%s\n",
		    vcc_table->param->pcr01, vcc_table->param->scr01,
		    vcc_table->param->mbs01, vcc_table->param->pcr0,
		    vcc_table->param->scr0, vcc_table->param->mbs0,
		    vcc_table->param->cdvt,
		    vcc_table->param->act > 0 ? "tag" : "drop",
		    vcc_table->param->aal5epd > 0 ? "yes" : "no");
	}
	else
	  sprintf(output_buf,
		  "\nPCR01\nSCR01\nMBS01\nPCR0\nSCR0\nMBS0\nCDVT\nACT\nAAL5EPD\n");

	fl_set_object_label(swview_status->status->swview_upc_text, output_buf);

	/* start polling timer */
	fl_set_timer(swview_status->status->swview_polling_timer,
		     TRAFFIC_POLLING_TIMER);

	/* show fd_swview_status */
	fl_show_form(swview_status->status->swview_status,
		     FL_PLACE_MOUSE, FL_TRANSIENT, "Status");

	fl_activate_all_forms();
    }
}

void
swview_polling_act(FL_OBJECT *ob, long data)
{
    int i;
    u_int prev_cells, max_bounds;
    char output_buf[BUFSIZ];

    swview_status_t *swview_status;

    /* search parent status object */

    swview_status = swview_search_parent_stauts(ob);

    /* rotate statistics */
    prev_cells = swview_status->cells;
    swctl_mon(swview_status->window->sw, swview_status->vcc_table->in,
	      swview_status->vcc_table->out, &(swview_status->cells));

    fl_freeze_form(swview_status->status->swview_status);

    /* clear chart */
    fl_clear_chart(swview_status->status->swview_traffic_chart);

    /* rotate items */
    for(i = 0; i < POLLING_ITEMS - 1; i++) {
	swview_status->items[i] = swview_status->items[i+1];
	swview_status->times[i] = swview_status->times[i+1];
    }
    swview_status->items[i] = swview_status->cells - prev_cells;
    swview_status->times[i] = swview_status->timer++;

    swview_status->max_cells = (swview_status->items[i] >
				swview_status->max_cells)
      ? swview_status->items[i]
	: swview_status->max_cells;
    max_bounds = swview_status->max_cells -
      (swview_status->max_cells % 1000) + 1000;


    /* rewrite attribute swview_status */
    sprintf(output_buf, "%u\ncells/sec", max_bounds);
    fl_set_object_label(swview_status->status->swview_maxcells_text,
			output_buf);
    sprintf(output_buf, "%u\ncells/sec", max_bounds/2);
    fl_set_object_label(swview_status->status->swview_centercells_text,
			output_buf);

    fl_set_chart_bounds(swview_status->status->swview_traffic_chart,
			0, max_bounds);

    for(i = 0; i < POLLING_ITEMS; i++) {
	if( swview_status->times[i] == -1 ) {
	    fl_add_chart_value(swview_status->status->swview_traffic_chart,
			       0, "", 1);
        }
	else if( swview_status->times[i] % 60 == 0 ) {
	    sprintf(output_buf, "%u:00", swview_status->times[i]/60);
	    fl_add_chart_value(swview_status->status->swview_traffic_chart,
			       swview_status->items[i], output_buf, 1);
	}
	else if( swview_status->times[i] % 30 == 0 ) {
	    sprintf(output_buf, "%u:30", swview_status->times[i]/60);
	    
	    fl_add_chart_value(swview_status->status->swview_traffic_chart,
			       swview_status->items[i], output_buf, 1);
	}
	else {
	    fl_add_chart_value(swview_status->status->swview_traffic_chart,
			       swview_status->items[i], "", 1);
	}
    }

    sprintf(output_buf, "Throughput\ncurrent: %u (c/s)", swview_status->items[POLLING_ITEMS - 1]);
    fl_set_object_label(swview_status->status->swview_nowcells_text,
			output_buf);

    fl_unfreeze_form(swview_status->status->swview_status);

    /* start polling timer */
    fl_set_timer(ob, TRAFFIC_POLLING_TIMER);
}

void
swview_dismiss_act(FL_OBJECT *ob, long data)
{
    
    swview_status_t *swview_status, *stat_cur, *stat_prev;

    /* search parent status object */
    swview_status = swview_search_parent_stauts(ob);

    /* close status window */
    fl_hide_form(swview_status->status->swview_status);
    fl_free_form(swview_status->status->swview_status);

    /* delete swview_status entry */
    for(stat_cur = swview_status_list, stat_prev = (swview_status_t *)NULL;
	stat_cur; stat_prev = stat_cur, stat_cur = stat_cur->next_status) {

	if ( stat_cur == swview_status ) {
	    free_vcc_table_list(swview_status->vcc_table);
	    if(stat_prev)
	      stat_prev->next_status = stat_cur->next_status;
	    else
	      swview_status_list = stat_cur->next_status;
	    free(stat_cur);
	}
    }
}

void
swview_operation_act(FL_OBJECT *ob, long data)
{
    int line, stat;
    int exit_flag = 0;
    char notion_buf[BUFSIZ];
    char output_buf[BUFSIZ];

    FD_swview_vcc *fd_vcc_window;
    FL_OBJECT *action_object;

    swview_window_t *swview_window;
    swview_vccline_t *swview_vccline;
    swview_status_t *swview_status;

    swctl_vcc_table_t *vcc_table;

    swctl_addr_t in, out;
    swctl_param_t param;

    /* search parent window object */
    swview_window = swview_search_parent_window(ob);

    /* search vccline object */
    line = fl_get_browser(ob);
    for(swview_vccline = swview_window->vccline_list; swview_vccline;
	swview_vccline = swview_vccline->next_line) {
	if(swview_vccline->line == line) {
	    vcc_table = swview_vccline->vcc_table;
	    break;
	}
    }

    /* the vcc is already monitoring */
    if( swview_search_status(swview_window->sw, vcc_table) ) {
	sprintf(notion_buf, "This VCC is monitoring now.\nPlease dismiss the monitoring window.");
	swview_alert_show(notion_buf);
    }
    else {

	fl_deactivate_all_forms();

	fd_vcc_window = create_form_swview_vcc();
        
	sprintf(output_buf, "on %s", swview_window->name);
	fl_set_object_label(fd_vcc_window->vcc_title, output_buf);
	fl_addto_choice(fd_vcc_window->choice, "CREATE");
	fl_addto_choice(fd_vcc_window->choice, "RELEASE");
	fl_addto_choice(fd_vcc_window->choice, "MODIFY");
	fl_addto_choice(fd_vcc_window->choice, "REDIRECT");
	fl_set_choice_fontsize(fd_vcc_window->choice, 14);
	fl_set_choice_fontstyle(fd_vcc_window->choice, 3);
	fl_set_choice_align(fd_vcc_window->choice, FL_ALIGN_CENTER);
	fl_set_choice(fd_vcc_window->choice, 1);

	swview_fill_vcc_paramters(swview_window, fd_vcc_window,
				  &(vcc_table->in), &(vcc_table->out),
				  vcc_table->param);

	fl_show_form(fd_vcc_window->swview_vcc,
		     FL_PLACE_MOUSE, FL_TRANSIENT, "VCC Operation");

	for( ; ; ) {
	    action_object = fl_do_forms();

	    if( action_object == fd_vcc_window->choice ) {
		switch(fl_get_choice(fd_vcc_window->choice)) {

		  case SWVIEW_CREATE_ACT:
		    fl_activate_object(fd_vcc_window->inPort);
		    fl_activate_object(fd_vcc_window->inVPI);
		    fl_activate_object(fd_vcc_window->inVCI);
		    fl_activate_object(fd_vcc_window->outPort);
		    fl_activate_object(fd_vcc_window->outVPI);
		    fl_activate_object(fd_vcc_window->outVCI);
		    fl_activate_object(fd_vcc_window->pcr01);
		    fl_activate_object(fd_vcc_window->scr01);
		    fl_activate_object(fd_vcc_window->mbs01);
		    fl_activate_object(fd_vcc_window->pcr0);
		    fl_activate_object(fd_vcc_window->scr0);
		    fl_activate_object(fd_vcc_window->mbs0);
		    fl_activate_object(fd_vcc_window->cdvt);
		    fl_activate_object(fd_vcc_window->upc_act_tag);
		    fl_activate_object(fd_vcc_window->upc_act_drop);
		    fl_activate_object(fd_vcc_window->vcc_aal5epd_on);
		    fl_activate_object(fd_vcc_window->vcc_aal5epd_off);
		    fl_activate_object(fd_vcc_window->use_upc_button);
		    break;

		  case SWVIEW_RELEASE_ACT:
		    swview_fill_vcc_paramters(swview_window, fd_vcc_window,
					      &(vcc_table->in),
					      &(vcc_table->out),
					      vcc_table->param);
		    
		    fl_deactivate_object(fd_vcc_window->inPort);
		    fl_deactivate_object(fd_vcc_window->inVPI);
		    fl_deactivate_object(fd_vcc_window->inVCI);
		    fl_deactivate_object(fd_vcc_window->outPort);
		    fl_deactivate_object(fd_vcc_window->outVPI);
		    fl_deactivate_object(fd_vcc_window->outVCI);
		    fl_deactivate_object(fd_vcc_window->pcr01);
		    fl_deactivate_object(fd_vcc_window->scr01);
		    fl_deactivate_object(fd_vcc_window->mbs01);
		    fl_deactivate_object(fd_vcc_window->pcr0);
		    fl_deactivate_object(fd_vcc_window->scr0);
		    fl_deactivate_object(fd_vcc_window->mbs0);
		    fl_deactivate_object(fd_vcc_window->cdvt);
		    fl_deactivate_object(fd_vcc_window->upc_act_tag);
		    fl_deactivate_object(fd_vcc_window->upc_act_drop);
		    fl_deactivate_object(fd_vcc_window->vcc_aal5epd_on);
		    fl_deactivate_object(fd_vcc_window->vcc_aal5epd_off);
		    fl_deactivate_object(fd_vcc_window->use_upc_button);
		    break;

		  case SWVIEW_MODIFY_ACT:
		    swview_fill_vcc_paramters(swview_window, fd_vcc_window,
					      &(vcc_table->in),
					      &(vcc_table->out),
					      vcc_table->param);
		    
		    fl_deactivate_object(fd_vcc_window->inPort);
		    fl_deactivate_object(fd_vcc_window->inVPI);
		    fl_deactivate_object(fd_vcc_window->inVCI);
		    fl_deactivate_object(fd_vcc_window->outPort);
		    fl_deactivate_object(fd_vcc_window->outVPI);
		    fl_deactivate_object(fd_vcc_window->outVCI);
		    fl_set_button(fd_vcc_window->use_upc_button, 1);
		    swview_vcc_change_upc(fd_vcc_window, 1);
		    break;

		  case SWVIEW_REDIRECT_ACT:
		    swview_fill_vcc_paramters(swview_window, fd_vcc_window,
					      &(vcc_table->in),
					      &(vcc_table->out),
					      vcc_table->param);
		    
		    fl_deactivate_object(fd_vcc_window->inPort);
		    fl_deactivate_object(fd_vcc_window->inVPI);
		    fl_deactivate_object(fd_vcc_window->inVCI);
		    fl_activate_object(fd_vcc_window->outPort);
		    fl_activate_object(fd_vcc_window->outVPI);
		    fl_activate_object(fd_vcc_window->outVCI);
		    fl_deactivate_object(fd_vcc_window->pcr01);
		    fl_deactivate_object(fd_vcc_window->scr01);
		    fl_deactivate_object(fd_vcc_window->mbs01);
		    fl_deactivate_object(fd_vcc_window->pcr0);
		    fl_deactivate_object(fd_vcc_window->scr0);
		    fl_deactivate_object(fd_vcc_window->mbs0);
		    fl_deactivate_object(fd_vcc_window->cdvt);
		    fl_deactivate_object(fd_vcc_window->upc_act_tag);
		    fl_deactivate_object(fd_vcc_window->upc_act_drop);
		    fl_deactivate_object(fd_vcc_window->vcc_aal5epd_on);
		    fl_deactivate_object(fd_vcc_window->vcc_aal5epd_off);
		    fl_deactivate_object(fd_vcc_window->use_upc_button);
		    break;

		  default:
		    break;
		}
	    }

	    else if( action_object == fd_vcc_window->ok_button ) {
		if(swview_get_vcc_paramters(swview_window,
					    fd_vcc_window,
					    &in, &out, &param)) {

		    switch(fl_get_choice(fd_vcc_window->choice)) {
			
		      case SWVIEW_CREATE_ACT:
			if( SWCTL_SUCCESS
			   == (stat = swctl_add(swview_window->sw,
						in, out, &param)) )
			  exit_flag++;
			else
			  swview_swctl_error(stat, swview_window->name);
			break;

		      case SWVIEW_RELEASE_ACT:
			if( SWCTL_SUCCESS
			   == (stat = swctl_delete(swview_window->sw,
						   in, out)) )
			  exit_flag++;
			else
			  swview_swctl_error(stat, swview_window->name);
			break;

		      case SWVIEW_MODIFY_ACT:
			if( SWCTL_SUCCESS
			   == (stat = swctl_modify(swview_window->sw,
						   in, out, &param)) )
			  exit_flag++;
			else
			  swview_swctl_error(stat, swview_window->name);
			break;

		      case SWVIEW_REDIRECT_ACT:
			if( SWCTL_SUCCESS
			   == (stat = swctl_redirect(swview_window->sw,
						     in, vcc_table->out,
						     out)) )
			  exit_flag++;
			else
			  swview_swctl_error(stat, swview_window->name);
			break;

		      default:
			break;
		    }
		    if(exit_flag)
		      break;
		}
	    }

	    else if(action_object == fd_vcc_window->clear_button) {

		switch(fl_get_choice(fd_vcc_window->choice)) {
			
		  case SWVIEW_CREATE_ACT:
		    fl_set_input(fd_vcc_window->inPort, "");
		    fl_set_input(fd_vcc_window->inVPI, "");
		    fl_set_input(fd_vcc_window->inVCI, "");
		    fl_set_input(fd_vcc_window->outPort, "");
		    fl_set_input(fd_vcc_window->outVPI, "");
		    fl_set_input(fd_vcc_window->outVCI, "");
		    fl_set_button(fd_vcc_window->use_upc_button, 0);
		    swview_vcc_change_upc(fd_vcc_window, 0);
		    fl_set_input(fd_vcc_window->pcr01, "");
		    fl_set_input(fd_vcc_window->scr01, "");
		    fl_set_input(fd_vcc_window->mbs01, "");
		    fl_set_input(fd_vcc_window->pcr0, "");
		    fl_set_input(fd_vcc_window->scr0, "");
		    fl_set_input(fd_vcc_window->mbs0, "");
		    fl_set_input(fd_vcc_window->cdvt, "");
		    fl_set_button(fd_vcc_window->upc_act_drop, 1);
		    fl_set_button(fd_vcc_window->vcc_aal5epd_off, 1);
		    break;
			
		  case SWVIEW_RELEASE_ACT:
		    break;

		  case SWVIEW_MODIFY_ACT:
		    fl_set_button(fd_vcc_window->use_upc_button, 0);
		    swview_vcc_change_upc(fd_vcc_window, 0);
		    fl_set_input(fd_vcc_window->pcr01, "");
		    fl_set_input(fd_vcc_window->scr01, "");
		    fl_set_input(fd_vcc_window->mbs01, "");
		    fl_set_input(fd_vcc_window->pcr0, "");
		    fl_set_input(fd_vcc_window->scr0, "");
		    fl_set_input(fd_vcc_window->mbs0, "");
		    fl_set_input(fd_vcc_window->cdvt, "");
		    fl_set_button(fd_vcc_window->upc_act_drop, 1);
		    fl_set_button(fd_vcc_window->vcc_aal5epd_off, 1);
		    break;

		  case SWVIEW_REDIRECT_ACT:
		    fl_set_input(fd_vcc_window->outPort, "");
		    fl_set_input(fd_vcc_window->outVPI, "");
		    fl_set_input(fd_vcc_window->outVCI, "");
		    break;

		  default:
		    break;
		}
	    }
	    else if(action_object == fd_vcc_window->cancel_button) {
		break;
	    }
	    else if(action_object == fd_vcc_window->use_upc_button) {

		if( 1 == fl_get_button(fd_vcc_window->use_upc_button)) 
		  swview_vcc_change_upc(fd_vcc_window, 1);
		else
		  swview_vcc_change_upc(fd_vcc_window, 0);
	    }
	}

	fl_hide_form(fd_vcc_window->swview_vcc);
	fl_free_form(fd_vcc_window->swview_vcc);
	fl_activate_all_forms();
    }
}

void
swview_add_act(FL_OBJECT *ob, long data)
{
}

void
swview_delete_act(FL_OBJECT *ob, long data)
{
}

void
swview_modify_act(FL_OBJECT *ob, long data)
{
}

void
swview_redirect_act(FL_OBJECT *ob, long data)
{
}

void
swview_refresh_act(FL_OBJECT *ob, long data)
{
    int val;
    
    swview_window_t	*swview_window;

    swctl_vcc_table_t	*vcc_table_list = (swctl_vcc_table_t *)NULL;
    swctl_upc_table_t	*upc_table_list = (swctl_upc_table_t *)NULL;

    /* serach parent window object */
    swview_window = swview_search_parent_window(ob);

    if( SNMP_SUCCESS != (val = swctl_stat(swview_window->sw,
					  &vcc_table_list,
					  &upc_table_list)) ) {
	swview_swctl_error(val, swview_window->name);
    }

    else {

	fl_freeze_form(swview_window->window->swview_switch);

	/* clear browser */
	fl_clear_browser(swview_window->window->swview_vcc_browser);
	free_vccline_list(swview_window->vccline_list);

	/* make vccline_list */
	swview_window->vccline_list = 
	  swview_set_vccline_list(swview_window->sw,
				  swview_window->window,
				  vcc_table_list);

	/* fill fd_swview_switch->swview_vcc_browser */
	swview_fill_vccline(swview_window->sw,
			    swview_window->window,
			    swview_window->vccline_list);

	fl_unfreeze_form(swview_window->window->swview_switch);

    }
    free_vcc_table_list(vcc_table_list);
    free_upc_table_list(upc_table_list);

}

void
swview_close_act(FL_OBJECT *ob, long data)
{
    int val;
    swview_window_t	*swview_window, *win_cur, *win_prev;
    swview_status_t	*stat_cur, *stat_prev;
    
    /* search parent window object */
    swview_window = swview_search_parent_window(ob);

    /* close status window & delete swview_status entry */
    for(stat_cur = swview_status_list, stat_prev = (swview_status_t *)NULL;
	stat_cur; stat_prev = stat_cur, stat_cur = stat_cur->next_status) {

	if ( stat_cur->window == swview_window ) {
	    fl_hide_form(stat_cur->status->swview_status);
	    fl_free_form(stat_cur->status->swview_status);
	    free_vcc_table_list(stat_cur->vcc_table);
	    if(stat_prev)
	      stat_prev->next_status = stat_cur->next_status;
	    else
	      swview_status_list = stat_cur->next_status;
	    free(stat_cur);
	}
    }

    if( SNMP_SUCCESS != (val = swctl_close(swview_window->sw)) ) {
	swview_swctl_error(val, swview_window->name);
    }

    /* close switch window */
    fl_hide_form(swview_window->window->swview_switch);
    fl_free_form(swview_window->window->swview_switch);

    free(swview_window->name);
    free_vccline_list(swview_window->vccline_list);

    /* delete swview_window entry */
    for(win_cur = swview_window_list, win_prev = (swview_window_t *)NULL;
	win_cur; win_prev = win_cur, win_cur = win_cur->next_window) {
	if( win_cur == swview_window ) {
	    if(win_prev)
	      win_prev->next_window = win_cur->next_window;
	    else
	      swview_window_list = win_cur->next_window;
	    free(win_cur);
	}
    }
}    

void
swview_trap_act(int sockfd, void *data)
{
    int buflen, sinSize;
    struct sockaddr_in sinRemote;

    switch_id_t sw;

    swview_window_t *swview_window;

    buflen = recvfrom( sockfd, &sw, sizeof(switch_id_t), 0,
		      (struct sockaddr *)&sinRemote, &sinSize );

    if( (swview_window = swview_search_window(sw)) )
      swview_refresh_act(swview_window->window->swview_refresh_button, 
			 0);
}

int
main(int argc, char *argv[], char *envp[])
{
    int arg, sockfd;
    char *conf_filename = (char *)NULL;
    struct sockaddr_in sinLocal, sinRemote;

    for( arg = 1; arg < argc; arg++ ) {
	if( argv[arg][0] == '-' ) {
	    switch(argv[arg][1]) {
	      case 'f':
		conf_filename = argv[++arg];
	      default:
		break;
	    }
	}
    }

    if(conf_filename)
      swview_conf_parse(conf_filename);
    else
      fprintf(stderr, "swview: no configuration file.(ignore)\n");

    fl_initialize(&argc, argv, 0, 0, 0);

    /* create fd_swview_main */
    fd_swview_main = create_form_swview_main();

    /* show fd_swview_main */
    fl_show_form(fd_swview_main->swview_main,
		 FL_PLACE_CENTER, FL_FULLBORDER,"SWView");

    /* set udp port & packet trap function */
    sockfd = socket( AF_INET, SOCK_DGRAM, 0 );

    sinLocal.sin_family = AF_INET;
    sinLocal.sin_addr.s_addr = htonl(INADDR_ANY);

    if( getenv("SWVIEW_PORT") )
      sinLocal.sin_port = htons(atoi(getenv("SWVIEW_PORT")));
    else
      sinLocal.sin_port = htons(DEFAULT_SWVIEW_PORT);

    bind( sockfd, (struct sockaddr *)&sinLocal, sizeof( struct
						       sockaddr_in) );
    fl_add_io_callback(sockfd, FL_READ, swview_trap_act,
		       (void *)NULL);

    /* do loop */
    fl_do_forms();
    exit(0);
}
